var jwt = require('jsonwebtoken');
module.exports = {
    name: 'auth',
    policy: (actionParams) => {
        return (req, res, next) => {
            if (req.method !== 'OPTIONS') {
                let authHeader = req.headers.authorization;
                let jwtToken, authInfo = '';
                if (authHeader && authHeader.split(" ")[0] === "Token") {
                    jwtToken = authHeader.split(" ")[1];
                } else {
                    if (actionParams.rule != 'optional') {
                        return res.status(401).send({
                            "status": "FAILURE",
                            "data": {
                                "code": "ERR_UNAUTHORIZED",
                                "message": "Token missing",
                                "description": "",
                                "errors": []
                            }
                        });
                    }
                }
                if (actionParams.rule == 'required') {
                    try {
                        console.log('required');
                        authInfo = jwt.verify(jwtToken, actionParams.jwtSecret);
                        console.log(authInfo);
                    } catch (error) {
                        return res.status(401).send({
                            "status": "FAILURE",
                            "data": {
                                "code": "ERR_UNAUTHORIZED",
                                "message": error.message,
                                "description": "",
                                "errors": []
                            }
                        });
                    }
                } else if (actionParams.rule == 'requiredIgnoreExpiry') {
                    try {
                        authInfo = jwt.verify(jwtToken, actionParams.jwtSecret, {ignoreExpiration: true});
                    } catch (error) {
                        return res.status(401).send({
                            "status": "FAILURE",
                            "data": {
                                "code": "ERR_UNAUTHORIZED",
                                "message": error.message,
                                "description": "",
                                "errors": []
                            }
                        });
                    }
                }
                if (authInfo != '') {
                    req.headers.authInfo = JSON.stringify(authInfo);
                }
            }
            next();
        }
    },
    schema: { // This is for Admin API to validate params
        $id: "http://express-gateway.io/models/applications.json",
        type: 'object',
        properties: {
            jwtSecret: {
                title: 'jwt secret',
                description: 'key used to sign jwt secret',
                type: 'string'
            },
            rule: {
                title: 'auth rule',
                description: 'specifies if auth is required, optional or required without considering expiry',
                type: 'string'
            }
        }
    }
}